==> kom%2B%2B-0.3.tar.gz: kom++-0.3/tcl-clients/nilkom2.tcl <==
# Some miscellaneous code for the nilkom client.
#
# $Id: nilkom2.tcl,v 1.30 1994/08/10 07:25:04 inge Exp $
# Copyright (C) 1994  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


# global variables (shared)
#  review_list   - list of textnos to review

proc read_text {tno} {
    global to_mark_as_read

    nilkom_page register s "**MER** (SPC, RET, s, q): "

    print_text $tno
    switch -glob [nilkom_page flush] {
	ready  {
	    lappend to_mark_as_read $tno
	}
	key*   {
	    # The user pressed "s". 
	    nilkom_page discard
	    nilkom_page acknowledge_key
	    nilkom_page sync
	    lappend to_mark_as_read $tno
	    print "Texten kommer att lsmarkeras om du inte ger kommandot\n"
	    print "  lsmarkera inte\n"
	}
	terminate {
	    nilkom_page sync
	    print "Text $tno lsmarkeras inte\n"
	}
	* {
	    print "WNG: Internal error: Strange return value from sync.\n"
	}
    }
}

proc k {{parent senaste}} {
	global inhibit_prompt
	global senaste

	if {[string compare $parent senaste] == 0} {
            start_comment $senaste
	} else {
            start_comment $parent
	}
	set inhibit_prompt 1
}
	
proc next_conference {} {
    set new_conf [kom_unread goto_next_conf]
    print [format "Du hamnade i %s (%d olsta)\n" \
	       [kom_conference name $new_conf] \
	       [lindex [kom_unread unread_texts_in_current] 1]]
}

proc goto_named_conference {name} {
    set confno [select_unique_any "Vilket mte vill du g till? " $name]
    # FIXME: Kontrollera redan hr om personen r med i mtet eller ej.
    #        Klienten vet ju vilka mten personen r med i.
    # FIXME: Om man inte r med i mtet ska man f ett erbjudande om
    #        att g med.
    if {$confno == ""} {
	if {[kom_unread current_conf]} {
	    print [format "Du str kvar i %s (%d olsta)\n" \
		       [kom_conference name [kom_unread current_conf]] \
		       [lindex [kom_unread unread_texts_in_current] 1]]
	} else {
	    if {1} {
		print "Du knner dig frnvarande.\n"
	    } else {
		print "Du r inte nrvarande i ngot mte.\n"
	    }
	}
    } else {
	set failed 0
	safe_eval {kom_unread goto_conf $confno} {
	    {
		{KOM FAILED-CALL}
		{
		    set failed 1
		}
	    }
	}
	if {$failed} {
	    # FIXME: Kontrollera returvrdet ovan.  Varfr
	    #        fick personen inte g till mtet?
	    print [format "Du kan inte g till mte %d (%s)\n" \
		       $confno [kom_conference name $confno]]
	    print "Antagligen r du inte medlem i mtet.\n"
	    print "Anvnd kommandot \"medlem\" fr att g med i mtet.\n"
	} else {
	    print [format "%s (%d olsta)\n" \
		       [kom_conference name $confno] \
		       [lindex [kom_unread unread_texts_in_current] 1]]
	}
    }
}

proc join_conference {name} {
    set confno [select_unique_any "Vilket mte vill du bli medlem i? " $name]
    if {$confno == ""} {
	print "Inget.\n"
	return
    }
    # FIXME: Anvndaren vill kanske ha andra prioriteter...

    set failed 0
    safe_eval {kom_unread join_conf $confno 99 1} {
	{
	    {KOM FAILED-CALL}
	    {
		set failed 1
	    }
	}
    }
    if {$failed} {
	print "Du kunde inte bli medlem i mtet.  Denna version av nilkom\n"
	print "r inte smart nog att bertta varfr; antagligen r\n"
	print "[kom_conference name $confno] ett slutet mte\n"
	print "eller en persons brevlda.  Elisp-klienten r bttre p\n"
	print "att diagnosticera fel.\n"
    } else {
	print "Du r nu medlem i [kom_conference name $confno].\n"
	print "nilkom har gett det prioritet 99, och det finns nst frst\n"
	print "p din mteslista.  nilkom kommer inte att ge mtet rtt\n"
	print "prioritet och placering denna session.\n"
    }
}

proc select_unique_any {prompt name} {
    set confno ""
    while {$confno == ""} {
	# Don't ask for the name if a name was supplied to this function.
	if {$name == ""} {
	    set name [getstr $prompt]
	}
	if {$name == ""} {
	    return ""
	}
	if {[regexp {^p[ \t]+([0-9]+)$} $name dummy confno]} {
	    safe_eval {kom_person number $confno} {
		{
		    {KOM BADPERS}
		    {
			print "Person nummer $confno finns inte\n"
			set confno ""
		    }
		}
	    }
	} elseif {[regexp {^m[ \t]+([0-9]+)$} $name dummy confno]} {
	    safe_eval {kom_conference number $confno} {
		{
		    {KOM BADCONF}
		    {
			print "Mte nummer $confno finns inte\n"
			set confno ""
		    }
		}
	    }
	} else {
	    set matches [kom_session lookup_name $name]
	    if {[llength $matches] == 0} {
		# No match
		set confno ""
		print "Det finns inget mte som matchar $name\n"
	    } elseif {[llength $matches] == 1} {
		set confno [microconf confno [lindex $matches 0]]
	    } else {
		set confno [select_a_match $matches]
	    }
	}
	# Make sure the second and subseqent loops ask the user for the name.
	set name ""
    }

    return $confno
}

proc reject {} {
    global to_mark_as_read

    if {[llength $to_mark_as_read]} {
	print "Text $to_mark_as_read lsmarkeras ej.\n"
	set to_mark_as_read {}
    } else {
	print "Inga texter var kade fr lsmarkering.\n"
    }
}

proc quick_unread {} {
    set total 0

    print "Du har olsta inlgg i fljande mten:\n\n"
    foreach conf [kom_unread unread_texts] {
        set confno [lindex $conf 0]
        set min [lindex $conf 1]
        set max [lindex $conf 2]

	if {$max == $min} {
	    print [format "     %6d %s\n" $min \
		[kom_conference name $confno]]
	} else {
	    print [format "%4d-%6d %s\n" $min $max \
		[kom_conference name $confno]]
	}
	incr total $min
    }
    print "\nTotalt minst $total olsta\n\n\n"
}

proc unread {} {
    set unread_confs [llength [kom_unread unread_texts]]
    set neednewline 0
    while {[kom_unread prefetch_map 0 20 $unread_confs 70 0] > 0} {
    }
    kom_unread prefetch_map 0 0 0 10 10
    quick_unread
}

proc keypress {n} {
    global keyhandler

    nilkom_page acknowledge
    $keyhandler $n
}

proc timeout {} {
    global prefetch_state

    kom_session handle_async
    switch $prefetch_state {
	notyet
	    {}
	0
	    {
    		if {[kom_unread prefetch_map 1000 1000 \
			 [llength [kom_unread unread_texts]] 2 0] == 0} {
		    set prefetch_state 1
		}
	    }
	1
	    {
		kom_unread prefetch 3 0 0 2 0
	    }
    }

    return 0
}

proc trim_review_list {} {
    global review_list

    set f 1
    while {$f==1} {
	set f 0
	if {[llength $review_list] > 0} {
	    safe_eval {kom_text author [lindex $review_list 0]} {
		{
		    {KOM BADTEXT}
		    {
			set review_list [lrange $review_list 1 end]
			set f 1
		    }
		}
	    }
	}
    }
}

proc review_next {} {
    global review_list

    if {[llength $review_list] == 0} {
	print "Det finns inget att se hr.\n"
    } else {
	print_text [lindex $review_list 0]
	set review_list [lrange $review_list 1 end]
    }
}

proc review_commented {tno} {
    global review_list

    set l {}
    set i 0
    while {$i<[kom_text num_uplinks $tno]} {
	lappend l [uplink parent [kom_text uplink $tno $i]]
	incr i
    }

    set review_list "$l $review_list"
    trim_review_list
    review_next
}

proc review_all_comments {tno} {
    global review_list

    set l {}

    set i 0
    while {$i<[kom_text num_footnotes $tno]} {
	lappend l [kom_text footnote $tno $i]
	incr i
    }

    set i 0
    while {$i<[kom_text num_comments $tno]} {
	lappend l [kom_text comment $tno $i]
	incr i
    }

    set review_list "$l $review_list"
    trim_review_list
    review_next
}

proc review_presentation {name} {
    set confno [select_unique_any "Vad vill du se presentationen fr? " $name]
    if {$confno == ""} {
	print "Du vill visst inte se ngon presentation.\n"
	return
    }

    set pres [kom_conference presentation $confno]
    if {$pres == 0} {
	print "[kom_conference name $confno] har ingen presentation.\n"
    } else {
	print_text $pres
    }
}


proc mark {tno type} {
    safe_eval {
	kom_text mark $tno $type
	print "Text $tno markerad med markeringstyp $type.\n"
    } {
	{{KOM BADTEXT} {print "Det finns ingen sdan text.\n"}}
    }
}

proc unmark {tno} {
    safe_eval {
	kom_text unmark $tno
	print "Text $tno avmarkerad.\n"
    } {
	{{KOM BADTEXT} {print "Det finns ingen sdan text.\n"}}
    }
}

proc help {} {
    global nilkom_version

    print "Vlkommen till nilkom!\n"
    print "\n"
    print "Trycker RETURN fr att utfra defaultkommandot.\n"
    print "Tryck Ctrl-C fr att avsluta nilkom.\n"
    print {Skriv "lista kommandon" fr att se vad du kan gra.}
    print "\nSkicka buggrapporter till ceder@lysator.liu.se.\n"
    print "Ha tlamod med att mnga finesser och kommandon saknas.\n"
    print "Detta r nilkom $nilkom_version\n"
}

proc who_is_on_small {} {
    set people [kom_session who_is_on] 
    foreach session $people {
	set line [format "%4d %-38.38s %-35.35s\n" \
		      [lindex $session 2] \
		      [safe_eval {kom_conference name [lindex $session 0]} {
			  {{KOM BADCONF 0 8} {format "oinloggad person"}}
			  {{KOM BADCONF} {format "hemlig person"}}}] \
		      [safe_eval {kom_conference name [lindex $session 1]} {
			  {{KOM BADCONF 0 8}
			      {format "ej nrvarande i ngot mte"}}
			  {{KOM BADCONF} {format "hemligt mte"}}}]]
	if {[print $line] == "terminate"} {
	    return
	}
    }
    print "Totalt [llength $people] personer.\n"
}

proc who_is_on_verbose {} {
    set people [kom_session who_is_on] 
    foreach session $people {
	set line [format "%4d %-38.38s %-35.35s\n" \
		      [lindex $session 2] \
		      [safe_eval {kom_conference name [lindex $session 0]} {
			  {{KOM BADCONF 0 8} {format "oinloggad person"}}
			  {{KOM BADCONF} {format "hemlig person"}}}] \
		      [safe_eval {kom_conference name [lindex $session 1]} {
			  {{KOM BADCONF 0 8}
			      {format "ej nrvarande i ngot mte"}}
			  {{KOM BADCONF} {format "hemligt mte"}}}]]
	if {[print $line] == "terminate"} {
	    return
	}
	set line [format "     %-38.38s %-35.35s\n" \
		      [lindex $session 4] [lindex $session 3]]
	if {[print $line] == "terminate"} {
	    return
	}
    }
    print "Totalt [llength $people] personer.\n"
}    

proc list_conferences {} {
    set microconflist [filter-confs [kom_session lookup_name ""]]
    set num_confs 0
    foreach mc $microconflist {
	set confno [microconf confno $mc]
	set line [format "%5d %s\n" $confno [kom_conference name $confno]]
	if {[print $line] == "terminate"} {
	    return
	}
	incr num_confs
    }
    print "\nTotalt $num_confs mten\n"
}


proc list_persons {} {
    set microconflist [filter-persons [kom_session lookup_name ""]]
    set num_confs 0
    foreach mc $microconflist {
	set confno [microconf confno $mc]
	set line [format "%5d %s\n" $confno [kom_conference name $confno]]
	if {[print $line] == "terminate"} {
	    return
	}
	incr num_confs
    }
    print "\nTotalt $num_confs personer\n"
}

proc send_message_command {name} {
    set persno [select_unique_person \
		    "Skicka personligt direktmeddelande till: " $name]
    if {$persno == ""} {
	print "Inget meddelande skickat.\n"
	return
    }
    print "Skicka meddelande till [kom_conference name $persno].\n"
    set msg [getstr "Meddelande: "]
    if {$msg == ""} {
	print "Inget meddelande skickat.\n"
	return
    }
    safe_eval {
	kom_session send_message $persno $msg
	print "Meddelande snt till [kom_conference name $persno].\n"
    } {
	{{KOM FAILED-CALL} {
	    print "Du kunde inte skicka meddelandet.  "
	    print "Mottagare r kanske oinloggad.\n"
	}}
    }
}

proc send_alert_message_command {} {
    print "Skicka alarmmeddelande till alla inloggade.\n"
    print "Somliga tycker inte om att f alarmmedelanden.  Tryck RETURN\n"
    print "utan att skriva ngot fr att avbryta.\n"
    set msg [getstr "Meddelande: "]
    if {$msg == ""} {
	print "Inget meddelande skickat.\n"
	return
    }
    safe_eval {
	kom_session send_message 0 $msg
	print "Alarmmeddelande snt till alla.\n"
    } {
	{{KOM FAILED-CALL} {
	    print "Mrkligt.  Du kunde inte skicka meddelandet.\n"
	}}
    }
}

proc quit_command {} {
    global quit_flag
    global inhibit_prompt

    set quit_flag 1
    set inhibit_prompt 1
    print "\nTack fr den hr gngen!\n"
}

==> kom%2B%2B-0.5.tar.gz: kom++-0.5/tcl-clients/nilkom2.tcl <==
# Some miscellaneous code for the nilkom client.
#
# $Id: nilkom2.tcl,v 1.36 1995/12/30 16:28:46 ceder Exp $
# Copyright (C) 1994  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


# global variables (shared)
#  review_list   - list of textnos to review

proc read_text {tno} {
    global to_mark_as_read

    nilkom_page register s "**MER** (SPC, RET, s, q): "

    print_text $tno
    switch -glob [nilkom_page flush] {
	ready  {
	    lappend to_mark_as_read $tno
	}
	key*   {
	    # The user pressed "s". 
	    nilkom_page discard
	    nilkom_page acknowledge_key
	    nilkom_page sync
	    lappend to_mark_as_read $tno
	    print "Texten kommer att lsmarkeras om du inte ger kommandot\n"
	    print "  lsmarkera inte\n"
	}
	terminate {
	    nilkom_page sync
	    print "Text $tno lsmarkeras inte\n"
	}
	* {
	    print "WNG: Internal error: Strange return value from sync.\n"
	}
    }
}

proc k {{parent senaste}} {
	global senaste

	if {[string compare $parent senaste] == 0} {
            start_comment $senaste
	} else {
            start_comment $parent
	}
}
	
proc next_conference {} {
    set new_conf [kom_unread goto_next_conf]
    print [format "Du hamnade i %s (%d olsta)\n" \
	       [kom_conference name $new_conf] \
	       [lindex [kom_unread unread_texts_in_current] 1]]
}

proc goto_named_conference {name} {
    set confno [select_unique_any "Vilket mte vill du g till? " $name]
    # FIXME: Kontrollera redan hr om personen r med i mtet eller ej.
    #        Klienten vet ju vilka mten personen r med i.
    # FIXME: Om man inte r med i mtet ska man f ett erbjudande om
    #        att g med.
    if {$confno == ""} {
	if {[kom_unread current_conf]} {
	    print [format "Du str kvar i %s (%d olsta)\n" \
		       [kom_conference name [kom_unread current_conf]] \
		       [lindex [kom_unread unread_texts_in_current] 1]]
	} else {
	    if {1} {
		print "Du knner dig frnvarande.\n"
	    } else {
		print "Du r inte nrvarande i ngot mte.\n"
	    }
	}
    } else {
	set failed 0
	safe_eval {kom_unread goto_conf $confno} {
	    {
		{KOM FAILED-CALL}
		{
		    set failed 1
		}
	    }
	}
	if {$failed} {
	    # FIXME: Kontrollera returvrdet ovan.  Varfr
	    #        fick personen inte g till mtet?
	    print [format "Du kan inte g till mte %d (%s)\n" \
		       $confno [kom_conference name $confno]]
	    print "Antagligen r du inte medlem i mtet.\n"
	    print "Anvnd kommandot \"medlem\" fr att g med i mtet.\n"
	} else {
	    print [format "%s (%d olsta)\n" \
		       [kom_conference name $confno] \
		       [lindex [kom_unread unread_texts_in_current] 1]]
	}
    }
}

proc join_conference {name} {
    set confno [select_unique_any "Vilket mte vill du bli medlem i? " $name]
    if {$confno == ""} {
	print "Inget.\n"
	return
    }
    # FIXME: Anvndaren vill kanske ha andra prioriteter...

    set failed 0
    safe_eval {kom_unread join_conf $confno 99 1} {
	{
	    {KOM FAILED-CALL}
	    {
		set failed 1
	    }
	}
    }
    if {$failed} {
	print "Du kunde inte bli medlem i mtet.  Denna version av nilkom\n"
	print "r inte smart nog att bertta varfr; antagligen r\n"
	print "[kom_conference name $confno] ett slutet mte\n"
	print "eller en persons brevlda.  Elisp-klienten r bttre p\n"
	print "att diagnosticera fel.\n"
    } else {
	print "Du r nu medlem i [kom_conference name $confno].\n"
	print "nilkom har gett det prioritet 99, och det finns nst frst\n"
	print "p din mteslista.  nilkom kommer inte att ge mtet rtt\n"
	print "prioritet och placering denna session.\n"
    }
}

proc select_unique_any {prompt name} {
    set confno ""
    while {$confno == ""} {
	# Don't ask for the name if a name was supplied to this function.
	if {$name == ""} {
	    set name [getstr $prompt]
	}
	if {$name == ""} {
	    return ""
	}
	if {[regexp {^p[ \t]+([0-9]+)$} $name dummy confno]} {
	    safe_eval {kom_person number $confno} {
		{
		    {KOM BADPERS}
		    {
			print "Person nummer $confno finns inte\n"
			set confno ""
		    }
		}
	    }
	} elseif {[regexp {^m[ \t]+([0-9]+)$} $name dummy confno]} {
	    safe_eval {kom_conference number $confno} {
		{
		    {KOM BADCONF}
		    {
			print "Mte nummer $confno finns inte\n"
			set confno ""
		    }
		}
	    }
	} else {
	    set matches [kom_session lookup_name $name]
	    if {[llength $matches] == 0} {
		# No match
		set confno ""
		print "Det finns inget mte som matchar $name\n"
	    } elseif {[llength $matches] == 1} {
		set confno [microconf confno [lindex $matches 0]]
	    } else {
		set confno [select_a_match $matches]
	    }
	}
	# Make sure the second and subseqent loops ask the user for the name.
	set name ""
    }

    return $confno
}

proc reject {} {
    global to_mark_as_read

    if {[llength $to_mark_as_read]} {
	print "Text $to_mark_as_read lsmarkeras ej.\n"
	set to_mark_as_read {}
    } else {
	print "Inga texter var kade fr lsmarkering.\n"
    }
}

proc quick_unread {} {
    set total 0

    print "Du har olsta inlgg i fljande mten:\n\n"
    foreach conf [kom_unread unread_texts] {
        set confno [lindex $conf 0]
        set min [lindex $conf 1]
        set max [lindex $conf 2]

	if {$max == $min} {
	    print [format "     %6d %s\n" $min \
		[kom_conference name $confno]]
	} else {
	    print [format "%4d-%6d %s\n" $min $max \
		[kom_conference name $confno]]
	}
	incr total $min
    }
    print "\nTotalt minst $total olsta\n\n\n"
}

proc unread {} {
    set unread_confs [llength [kom_unread unread_texts]]
    set neednewline 0
    while {[kom_unread prefetch 0 0 0 0 20 $unread_confs 70] > 0} {
    }
    quick_unread
}

proc timeout {} {
    global prefetch_state
    global nilkom_prompt
    global prompt_state
    global idling_disabled

    kom_session handle_async
    switch $prefetch_state {
	notyet	
	{
	}
	0
	    {
    		if {[kom_unread prefetch 0 0 0 1000 1000 \
			 [llength [kom_unread unread_texts]] 2] == 0} {
		    set prefetch_state 1
		}
	    }
	1
	    {
		kom_unread prefetch 3 0 0 0 0 0 2
	    }
    }

    if {$prompt_state != "" && $idling_disabled == 0} {
	nilkom_page remove " "
	nilkom_page remove $prompt_state
	switch -exact -- $prompt_state {
	    - { set prompt_state "\\" }
	    \\ { set prompt_state "|" }
	    | { set prompt_state "/" }
	    / { set prompt_state "-" }
	}
	print "$prompt_state "
    }

    return 0
}

proc trim_review_list {} {
    global review_list

    set f 1
    while {$f==1} {
	set f 0
	if {[llength $review_list] > 0} {
	    safe_eval {kom_text author [lindex $review_list 0]} {
		{
		    {KOM BADTEXT}
		    {
			set review_list [lrange $review_list 1 end]
			set f 1
		    }
		}
	    }
	}
    }
}

proc review_next {} {
    global review_list

    if {[llength $review_list] == 0} {
	print "Det finns inget att se hr.\n"
    } else {
	print_text [lindex $review_list 0]
	set review_list [lrange $review_list 1 end]
    }
}

proc review_commented {tno} {
    global review_list

    set l {}
    set i 0
    while {$i<[kom_text num_uplinks $tno]} {
	lappend l [uplink parent [kom_text uplink $tno $i]]
	incr i
    }

    set review_list "$l $review_list"
    trim_review_list
    review_next
}

proc review_all_comments {tno} {
    global review_list

    set l {}

    set i 0
    while {$i<[kom_text num_footnotes $tno]} {
	lappend l [kom_text footnote $tno $i]
	incr i
    }

    set i 0
    while {$i<[kom_text num_comments $tno]} {
	lappend l [kom_text comment $tno $i]
	incr i
    }

    set review_list "$l $review_list"
    trim_review_list
    review_next
}

proc review_presentation {name} {
    set confno [select_unique_any "Vad vill du se presentationen fr? " $name]
    if {$confno == ""} {
	print "Du vill visst inte se ngon presentation.\n"
	return
    }

    set pres [kom_conference presentation $confno]
    if {$pres == 0} {
	print "[kom_conference name $confno] har ingen presentation.\n"
    } else {
	print_text $pres
    }
}


proc mark {tno type} {
    safe_eval {
	kom_text mark $tno $type
	print "Text $tno markerad med markeringstyp $type.\n"
    } {
	{{KOM BADTEXT} {print "Det finns ingen sdan text.\n"}}
    }
}

proc unmark {tno} {
    safe_eval {
	kom_text unmark $tno
	print "Text $tno avmarkerad.\n"
    } {
	{{KOM BADTEXT} {print "Det finns ingen sdan text.\n"}}
    }
}

proc help {} {
    global nilkom_version

    print "Vlkommen till nilkom!\n"
    print "\n"
    print "Trycker RETURN fr att utfra defaultkommandot.\n"
    print "Tryck Ctrl-C fr att avsluta nilkom.\n"
    print {Skriv "lista kommandon" fr att se vad du kan gra.}
    print "\nSkicka buggrapporter till ceder@lysator.liu.se.\n"
    print "Ha tlamod med att mnga finesser och kommandon saknas.\n"
    print "Detta r nilkom $nilkom_version\n"
}

proc who_is_on_small {} {
    set people [kom_session who_is_on] 
    foreach session $people {
	set line [format "%4d %-38.38s %-35.35s\n" \
		      [lindex $session 2] \
		      [safe_eval {kom_conference name [lindex $session 0]} {
			  {{KOM BADCONF 0 8} {format "oinloggad person"}}
			  {{KOM BADCONF} {format "hemlig person"}}}] \
		      [safe_eval {kom_conference name [lindex $session 1]} {
			  {{KOM BADCONF 0 8}
			      {format "ej nrvarande i ngot mte"}}
			  {{KOM BADCONF} {format "hemligt mte"}}}]]
	if {[print $line] == "terminate"} {
	    return
	}
    }
    print "Totalt [llength $people] personer.\n"
}

proc who_is_on_verbose {} {
    set people [kom_session who_is_on] 
    foreach session $people {
	set line [format "%4d %-38.38s %-35.35s\n" \
		      [lindex $session 2] \
		      [safe_eval {kom_conference name [lindex $session 0]} {
			  {{KOM BADCONF 0 8} {format "oinloggad person"}}
			  {{KOM BADCONF} {format "hemlig person"}}}] \
		      [safe_eval {kom_conference name [lindex $session 1]} {
			  {{KOM BADCONF 0 8}
			      {format "ej nrvarande i ngot mte"}}
			  {{KOM BADCONF} {format "hemligt mte"}}}]]
	if {[print $line] == "terminate"} {
	    return
	}
	set line [format "     %-38.38s %-35.35s\n" \
		      [lindex $session 4] [lindex $session 3]]
	if {[print $line] == "terminate"} {
	    return
	}
    }
    print "Totalt [llength $people] personer.\n"
}    

proc list_conferences {} {
    set microconflist [filter-confs [kom_session lookup_name ""]]
    set num_confs 0
    foreach mc $microconflist {
	set confno [microconf confno $mc]
	set line [format "%5d %s\n" $confno [kom_conference name $confno]]
	if {[print $line] == "terminate"} {
	    return
	}
	incr num_confs
    }
    print "\nTotalt $num_confs mten\n"
}


proc list_persons {} {
    set microconflist [filter-persons [kom_session lookup_name ""]]
    set num_confs 0
    foreach mc $microconflist {
	set confno [microconf confno $mc]
	set line [format "%5d %s\n" $confno [kom_conference name $confno]]
	if {[print $line] == "terminate"} {
	    return
	}
	incr num_confs
    }
    print "\nTotalt $num_confs personer\n"
}

proc send_message_command {name} {
    set persno [select_unique_any \
		    "Skicka direktmeddelande till: " $name]
    if {$persno == ""} {
	print "Inget meddelande skickat.\n"
	return
    }
    print "Skicka meddelande till [kom_conference name $persno].\n"
    set msg [getstr "Meddelande: "]
    if {$msg == ""} {
	print "Inget meddelande skickat.\n"
	return
    }
    safe_eval {
	kom_session send_message $persno $msg
	print "Meddelande snt till [kom_conference name $persno].\n"
    } {
	{{KOM FAILED-CALL} {
	    print "Du kunde inte skicka meddelandet.  "
	    print "Mottagare r kanske oinloggad.\n"
	}}
    }
}

proc send_alert_message_command {} {
    print "Skicka alarmmeddelande till alla inloggade.\n"
    print "Somliga tycker inte om att f alarmmedelanden.  Tryck RETURN\n"
    print "utan att skriva ngot fr att avbryta.\n"
    set msg [getstr "Meddelande: "]
    if {$msg == ""} {
	print "Inget meddelande skickat.\n"
	return
    }
    safe_eval {
	kom_session send_message 0 $msg
	print "Alarmmeddelande snt till alla.\n"
    } {
	{{KOM FAILED-CALL} {
	    print "Mrkligt.  Du kunde inte skicka meddelandet.\n"
	}}
    }
}

proc receive_message_command {} {
    global last_message_reply_to
    global message_queue

    if {[llength $message_queue] == 0} {
	print "Mrkligt.  Det finns inget meddelande att ta emot.\n"
    } else {
	set msg [lindex $message_queue 0]
	set receiver [lindex $msg 0]
	set sender [lindex $msg 1]
	set message_queue [lrange $message_queue 1 end]
	set send_name [kom_conference name $sender]
	if {$receiver == 0} {
	    print "Alarmmeddelande frn $send_name:\n"
	} else {
	    set rcpt_name [kom_conference name $receiver]

	    print "Meddelande frn $send_name till $rcpt_name:\n\n"
	}
	print "[lindex $msg 2]\n\n"

	set me [lindex [kom_session who_am_i] 0]
	if {$me != $sender} {
	    if {$receiver == $me || $receiver == 0} {
		set last_message_reply_to $sender
		print "Svar till sndaren.\n"
	    } else {
		set last_message_reply_to $receiver
		print "Svar till alla mottagare.\n"
	    }
	}
    }
}

proc reply_to_message_command {} {
    global last_message_reply_to
 
    if {$last_message_reply_to == ""} {
	print "Mrkligt.  Det finns ingen att skicka svaret till.\n"
    } else {
	set rcpt $last_message_reply_to
	set last_message_reply_to ""
	print "Skicka meddelande till [kom_conference name $rcpt].\n"
	set msg [getstr "Meddelande: "]
	if {$msg == ""} {
	    print "Inget meddelande skickat.\n"
	    return
	}
	safe_eval {
	    kom_session send_message $rcpt $msg
	    print "Meddelande snt till [kom_conference name $rcpt].\n"
	} {
	    {
		{KOM FAILED-CALL} {
		    print "Du kunde inte skicka meddelandet.  "
		    print "Mottagare r kanske oinloggad.\n"
		}
	    }
	}
    }
}

proc text_msg_handler {rec send msg} {
    global message_queue
    global prompt_state

    lappend message_queue [list $rec $send $msg]
    if {$prompt_state != ""} {
	print "\n[calculate_prompt]"
    }
}

proc quit_command {} {
    global quit_flag

    set quit_flag 1
    print "\nTack fr den hr gngen!\n"
}
